@@ -11,7 +11,8 @@ class OdataNode(AbstractNode, abc.ABC):
 
     def __init__(self, service_url, auth: AuthBase = None):
         super(OdataNode, self).__init__()
-        self._service_url = service_url
+        self._service_url = service_url.replace('+odata', '') \
+            if '+odata' in service_url else service_url
         self._auth = auth
 
     def get_service_url(self) -> str:
