@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import uuid
 import io
+import datetime
 from drb import DrbNode
 from drb.path import ParsedPath
 from drb.predicat import Predicate
@@ -10,9 +11,9 @@ from requests.auth import AuthBase
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 from .odata_node import OdataNode
-from .odata_utils import req_svc_products, req_product_by_uuid, req_svc_count,\
-    req_product_download, req_product_attributes, ODataQueryPredicate, \
-    OdataServiceType, get_type_odata_svc
+from .odata_utils import req_svc_products, req_product_by_uuid, \
+    req_svc_count, req_product_download, req_product_attributes,\
+    ODataQueryPredicate, OdataServiceType, get_type_odata_svc
 from .exceptions import OdataRequestException
 
 
@@ -59,7 +60,7 @@ class ODataServiceNode(OdataNode):
 
     @property
     def children(self) -> List[DrbNode]:
-        return ProductList(self)
+        return ODataFilteredList(self)
 
     def get_attribute(self, name: str, namespace_uri: str = None) -> Any:
         raise DrbException('ODataNode has no attribute')
@@ -82,7 +83,7 @@ class ODataServiceNode(OdataNode):
         raise DrbException(f"ODataNot doesn't support {impl} implementation")
 
     def _get_named_child(self, name: str, namespace_uri: str = None,
-                         occurrence: Union[int, slice] = 0)\
+                         occurrence: Union[int, slice] = 0) \
             -> Union[DrbNode, List[DrbNode]]:
         if self.type_service == OdataServiceType.ONDA_DIAS:
             id_identifier = 'id'
@@ -135,8 +136,7 @@ class ODataServiceNode(OdataNode):
             if isinstance(key, str):
                 return self._get_named_child(key)
             if isinstance(key, ODataQueryPredicate):
-                return [ODataProductNode(self, p[id_identifier], data=p)
-                        for p in key.apply_query(self)]
+                return ODataFilteredList(self, key.filter, key.order)
         except DrbException as ex:
             raise KeyError from ex
         raise TypeError('Invalid type for a DrbNode bracket: '
@@ -154,6 +154,169 @@ class ODataServiceNode(OdataNode):
         return self.__retrieve_child(other)
 
 
+class ODataFilteredList(list):
+    _page_size = 100
+
+    def __init__(self, odata: OdataNode, qfilter: str = None,
+                 qorder: str = None):
+        super().__init__()
+        self._count = -1
+        self._page = None
+        self._skip = 0
+        self._odata = odata
+        default = self.__retrieve_default_criteria(odata)
+        self._filter = self.__prepare_filter(qfilter, default)
+        self._order = self.__prepare_order(qorder, default)
+
+    @classmethod
+    def __retrieve_default_criteria(cls, odata: OdataNode) -> str:
+        if odata.type_service == OdataServiceType.DHUS:
+            return 'CreationDate'
+        elif odata.type_service == OdataServiceType.ONDA_DIAS:
+            return 'creationDate'
+        elif odata.type_service == OdataServiceType.CSC:
+            return 'PublicationDate'
+        else:
+            raise DrbException('Unsupported OData service')
+
+    @classmethod
+    def __prepare_filter(cls, user_filter: str, default: str) -> str:
+        date = datetime.datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
+        if user_filter is None:
+            return f'{default} lt {date}'
+        if default in user_filter:
+            return user_filter
+        return f"({user_filter}) and {default} lt {date}"
+
+    @classmethod
+    def __prepare_order(cls, user_order: str, default: str) -> str:
+        if user_order is None:
+            return f'{default} desc'
+        return user_order
+
+    def __generate_product(self, data: dict) -> ODataProductNode:
+        if self._odata.type_service == OdataServiceType.ONDA_DIAS:
+            return ODataProductNode(self._odata, data['id'], data=data)
+        else:
+            return ODataProductNode(self._odata, data['Id'], data=data)
+
+    def __perform_query(self):
+        if self._count == -1:
+            buffer, self._count = req_svc_products(self._odata, count=True,
+                                                   filter=self._filter,
+                                                   order=self._order,
+                                                   skip=self._skip,
+                                                   top=self._page_size)
+        else:
+            buffer = req_svc_products(self._odata, filter=self._filter,
+                                      order=self._order, skip=self._skip,
+                                      top=self._page_size)
+        self._page = [self.__generate_product(e) for e in buffer]
+
+    def __compute_index(self, item: Union[int, slice]) -> Tuple[int, int]:
+        if isinstance(item, int):
+            if item < 0:
+                return item + len(self), -1
+            return item, -1
+        # item is a slice
+        start = item.start if item.start is not None else 0
+        if start < 0:
+            start = start + len(self)
+        stop = item.stop if item.stop is not None else len(self)
+        if stop < 0:
+            stop = stop + len(self)
+        return start, stop
+
+    def append(self, obj: Any) -> None:
+        raise NotImplementedError
+
+    def clear(self) -> None:
+        raise NotImplementedError
+
+    def copy(self) -> List[ODataProductNode]:
+        raise NotImplementedError
+
+    def count(self, value: Any) -> int:
+        raise NotImplementedError
+
+    def extend(self, iterable: Iterable[ODataProductNode]) -> None:
+        raise NotImplementedError
+
+    def index(self, value: Any, start: int = ..., stop: int = ...) -> int:
+        raise NotImplementedError
+
+    def insert(self, index: int, obj: ODataProductNode) -> None:
+        raise NotImplementedError
+
+    def pop(self, index: int = ...) -> ODataProductNode:
+        raise NotImplementedError
+
+    def remove(self, value: Any) -> None:
+        raise NotImplementedError
+
+    def reverse(self) -> None:
+        raise NotImplementedError
+
+    def sort(self: List, *, key: None = ..., reverse: bool = ...) -> None:
+        raise NotImplementedError
+
+    def __getitem__(self, item):
+        if self._count == -1:
+            self.__perform_query()
+
+        if isinstance(item, int):
+            index, _ = self.__compute_index(item)
+            if index not in range(self._skip, self._skip + self._page_size):
+                page_number = index // self._page_size
+                self._skip = page_number * self._page_size
+                self.__perform_query()
+            return self._page[index % self._page_size]
+        elif isinstance(item, slice):
+            start, stop = self.__compute_index(item)
+            result = []
+            total = stop - start
+            page = 0
+            while len(result) < total:
+                skip = (page * self._page_size) + start
+                top = min(total - len(result), self._page_size)
+                resp = req_svc_products(self._odata, filter=self._filter,
+                                        order=self._order, skip=skip, top=top)
+                result.extend(resp)
+                page += 1
+            return [self.__generate_product(e) for e in result]
+        else:
+            raise KeyError(f'Invalid key: {type(item)}')
+
+    def __iter__(self):
+        def generator(qfilter, qorder, count, page_size):
+            page_num = 0
+            buffer = None
+            for i in range(count):
+                if i % page_size == 0:
+                    skip = page_size * page_num
+                    buffer = req_svc_products(self._odata, filter=qfilter,
+                                              order=qorder, skip=skip,
+                                              top=page_size)
+                    page_num += 1
+                idx = i % page_size
+                yield self.__generate_product(buffer[idx])
+        return generator(self._filter, self._order, len(self), self._page_size)
+
+    def __len__(self):
+        if self._count == -1:
+            self.__perform_query()
+        return self._count
+
+    def __contains__(self, item):
+        raise NotImplementedError
+
+    def __iadd__(self, other):
+        raise NotImplementedError
+
+    def __imul__(self, other):
+        raise NotImplementedError
+
+
 class ProductList(list):
     """
     Specific read only list to access to OData CSC Product entities:
@@ -241,7 +404,9 @@ class ProductList(list):
 
 
 class ODataProductNode(OdataNode):
-    def __init__(self, source: Union[str, OdataNode], product_uuid: str,
+    def __init__(self, source: Union[str, OdataNode],
+                 product_uuid: str = None,
+                 data: Dict = None,
                  **kwargs):
         self._type = OdataServiceType.UNKNOWN
 
@@ -250,15 +415,30 @@ class ODataProductNode(OdataNode):
                                                    source.get_auth())
             self._type = source.type_service
             self.__parent = source
+            self.__uuid = product_uuid
+        elif isinstance(data, Dict):
+            self.__parent = None
+            auth = kwargs['auth'] if 'auth' in kwargs.keys() else None
+            super(ODataProductNode, self).__init__(
+                source.split('/Products')[0],
+                auth
+            )
+            self._type = get_type_odata_svc(source.split('/Products')[0], auth)
+            self.__uuid = data.get('id') if \
+                self._type == OdataServiceType.ONDA_DIAS.value \
+                else data.get('Id')
         else:
             auth = kwargs['auth'] if 'auth' in kwargs.keys() else None
-            super(ODataProductNode, self).__init__(source, auth)
+            super(ODataProductNode, self).__init__(
+                source.split('/Products')[0],
+                auth
+            )
             self.__parent = None
+            self.__uuid = product_uuid
 
-        self.__uuid = product_uuid
         self.__path = ParsedPath(
             f'{self.get_service_url()}/Products({self.__uuid})')
-        self.__product = kwargs['data'] if 'data' in kwargs.keys() else None
+        self.__product = kwargs.get('data', None)
         self.__attr = None
 
     @property
