@@ -68,12 +68,15 @@ def get_type_odata_svc(service_url: str, auth: AuthBase = None) \
         -> OdataServiceType:
     """
     Retrieve with the given URL the OData service type (CSC or DHuS).
-    :param service_url: service URL
-    :type service_url: str
-    :param auth: (optional) authentication mechanism required by the service
-    :param auth: AuthBase
-    :returns: OdataServiceType value corresponding to service.
-    :rtype: OdataServiceType
+
+    Parameters:
+        service_url (str): OData service URL
+        auth (AuthBase): authentication mechanism required by the service
+                         (default ``None``)
+    Returns:
+        ODataServiceType: ODataServiceType corresponding to the service
+    Raises:
+        DrbException: if the OData schema ($metadata) cannot be done
     """
     try:
         url = f'{service_url}/$metadata'
@@ -87,7 +90,7 @@ def get_type_odata_svc(service_url: str, auth: AuthBase = None) \
         elif 'Ens'.lower() == ns.lower():
             return OdataServiceType.ONDA_DIAS
         return OdataServiceType.UNKNOWN
-    except (DrbException, ParseError):
+    except ParseError:
         return OdataServiceType.UNKNOWN
 
 
@@ -116,7 +119,7 @@ def req_svc_count(odata: OdataNode) -> int:
 def req_svc_products(odata: OdataNode, **kwargs) -> list:
     params = {'$format': 'json'}
     if 'filter' in kwargs.keys() and kwargs['filter'] is not None:
-        params['$filter'] = kwargs['filter']
+        params['$filter'] = kwargs['filter'].replace('\'', '%27')
     if 'search' in kwargs.keys() and kwargs['search'] is not None:
         params['$search'] = kwargs['search']
     if 'order' in kwargs.keys() and kwargs['order'] is not None:
@@ -125,11 +128,16 @@ def req_svc_products(odata: OdataNode, **kwargs) -> list:
         params['$skip'] = kwargs['skip']
     if 'top' in kwargs.keys() and kwargs['top'] is not None:
         params['$top'] = kwargs['top']
+    if 'count' in kwargs.keys() and kwargs['count']:
+        params['$count'] = 'true'
 
     query = '&'.join(map(lambda k: f'{k[0]}={k[1]}', params.items()))
     url = f'{odata.get_service_url()}/Products?{query}'
     node = DrbHttpNode(url, auth=odata.get_auth())
     data = http_node_to_json(node)
+    if 'count' in kwargs.keys() and kwargs['count']:
+        if '@odata.count' in data.keys():
+            return data['value'], data['@odata.count']
     return data['value']
 
 
@@ -171,36 +179,35 @@ def req_product_download(
 
 
 class ODataQueryPredicate(Predicate):
-    """This Predicate allows to customize the OData query request."""
+    """
+    This predicate allowing to customize an OData query request.
+    Customizable OData query elements:
+     - filter
+     - search
+     - orderby
+
+    Keyword Arguments:
+        filter (str): the OData filter query element
+        search (str): the OData search query element
+        order (str): the OData orderby query element
+    """
 
     def __init__(self, **kwargs):
-        """
-        ODataCustomQuery constructor
-
-        :key filter: OData query filter
-        :key search: OData query search
-        :key order: OData query orderby
-        :key skip: OData query skip
-        :key top: ODate query top
-        """
         self.__filter = kwargs['filter'] if 'filter' in kwargs.keys() else None
         self.__search = kwargs['search'] if 'search' in kwargs.keys() else None
         self.__order = kwargs['order'] if 'order' in kwargs.keys() else None
-        self.__skip = kwargs['skip'] if 'skip' in kwargs.keys() else None
-        self.__top = kwargs['top'] if 'top' in kwargs.keys() else None
+
+    @property
+    def filter(self) -> str:
+        return self.__filter
+
+    @property
+    def order(self) -> str:
+        return self.__order
+
+    @property
+    def search(self) -> str:
+        return self.__search
 
     def matches(self, key) -> bool:
         return False
-
-    def apply_query(self, node: OdataNode) -> List[dict]:
-        """
-        Performs the request with specific given query parameters.
-
-        :param node: OData service on which perform the request.
-        :type node: OdataNode
-        :returns: list of dictionary (JSON OData response)
-        :rtype: list
-        """
-        return req_svc_products(node, filter=self.__filter, order=self.__order,
-                                skip=self.__skip, top=self.__top,
-                                search=self.__search)
